searchData={"items":[{"type":"module","title":"Peri","doc":"Peri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema.\nIt provides a flexible and powerful way to define and validate data structures using schemas.\nThe library supports nested schemas, optional fields, custom validation functions, and various type constraints.","ref":"Peri.html"},{"type":"module","title":"Key Features - Peri","doc":"- **Simple and Nested Schemas**: Define schemas that can handle complex, nested data structures.\n- **Optional and Required Fields**: Specify fields as optional or required with type constraints.\n- **Custom Validation Functions**: Use custom functions to validate fields.\n- **Comprehensive Error Handling**: Provides detailed error messages for validation failures.\n- **Type Constraints**: Supports various types including enums, lists, maps, tuples, literals, and more.","ref":"Peri.html#module-key-features"},{"type":"module","title":"Usage - Peri","doc":"To define a schema, use the `defschema` macro. By default, all fields in the schema are optional unless specified otherwise.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    geolocation: {:tuple, [:float, :float]},\n    preferences: {:map, :string},\n    scores: {:map, :string, :integer},\n    status: {:literal, :active},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n```\n\nYou can then use the schema to validate data:\n\n```elixir\nuser_data = %{\n  name: \"John\", age: 30, email: \"john@example.com\",\n  address: %{street: \"123 Main St\", city: \"Somewhere\"},\n  tags: [\"science\", \"funky\"], role: :admin,\n  geolocation: {12.2, 34.2},\n  preferences: %{\"theme\" => \"dark\", \"notifications\" => \"enabled\"},\n  scores: %{\"math\" => 95, \"science\" => 92},\n  status: :active,\n  rating: 9\n}\n\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-usage"},{"type":"module","title":"Error Handling - Peri","doc":"Peri provides detailed error messages that include the path to the invalid data, the expected and actual values, and custom error messages for custom validations.","ref":"Peri.html#module-error-handling"},{"type":"module","title":"Schema Types - Peri","doc":"Peri supports the following schema types:\n\n- `:string`, `:integer`, `:float`, `:boolean`, `:atom`, `:map`, `:pid` - Basic types\n- `{:required, type}` - Mark a field as required\n- `{:list, type}` - List of elements of the given type\n- `{:map, type}` - Map with values of the given type\n- `{:map, key_type, value_type}` - Map with keys and values of specified types\n- `{:schema, schema}` - Explicitly tagged nested schema\n- `{:schema, map_schema, {:additional_keys, type}}` - Nested schema map, with extra entries validated using another type\n- `{:tuple, [type1, type2, ...]}` - Tuple with elements of specified types\n- `{:enum, [value1, value2, ...]}` - One of the specified values\n- `{:literal, value}` - Exactly matches the specified value\n- `{:either, {type1, type2}}` - Either type1 or type2\n- `{:oneof, [type1, type2, ...]}` - One of the specified types\n- `{:cond, condition, true_type, false_type}` - Conditional validation based on callback\n- `{:dependent, callback}` - Dynamic type based on callback result\n- Nested maps for complex structures","ref":"Peri.html#module-schema-types"},{"type":"module","title":"Callback Functions for :cond and :dependent - Peri","doc":"Both `:cond` and `:dependent` types support 1-arity and 2-arity callbacks:\n\n- **1-arity callbacks** receive the root data structure (backward compatible)\n- **2-arity callbacks** receive `(current, root)` where:\n  - `current` is the data at the current validation context (e.g., list element)\n  - `root` is the entire root data structure\n\nThis is especially useful when validating elements within lists:\n\n```elixir\ndefschema :parent, %{\n  items: {:list, %{\n    type: :string,\n    value: {:dependent, fn current, _root ->\n      case current.type do\n        \"number\" -> {:ok, :integer}\n        \"text\" -> {:ok, :string}\n        _ -> {:ok, :any}\n      end\n    end}\n  }}\n}\n```","ref":"Peri.html#module-callback-functions-for-cond-and-dependent"},{"type":"module","title":"Functions - Peri","doc":"- `validate/2` - Validates data against a schema.\n- `conforms?/2` - Checks if data conforms to a schema.\n- `validate_schema/1` - Validates the schema definition.\n- `generate/1` - Generates sample data based on schema (when StreamData is available).","ref":"Peri.html#module-functions"},{"type":"module","title":"Example - Peri","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\nend\n\nuser_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-example"},{"type":"type","title":"Peri.cond_def/0","doc":"","ref":"Peri.html#t:cond_def/0"},{"type":"function","title":"Peri.conforms?/3","doc":"Checks if the given data conforms to the specified schema.","ref":"Peri.html#conforms?/3"},{"type":"function","title":"Parameters - Peri.conforms?/3","doc":"- `schema`: The schema definition to validate against.\n  - `data`: The data to be validated.","ref":"Peri.html#conforms?/3-parameters"},{"type":"function","title":"Options - Peri.conforms?/3","doc":"- `:mode` - Validation mode. Can be `:strict` (default) or `:permissive`.\n    - `:strict` - Only fields defined in the schema are returned.\n    - `:permissive` - All fields from the input data are preserved.","ref":"Peri.html#conforms?/3-options"},{"type":"function","title":"Returns - Peri.conforms?/3","doc":"- `true` if the data conforms to the schema.\n  - `false` if the data does not conform to the schema.","ref":"Peri.html#conforms?/3-returns"},{"type":"function","title":"Examples - Peri.conforms?/3","doc":"iex> schema = %{name: :string, age: :integer}\n    iex> data = %{name: \"Alice\", age: 30}\n    iex> Peri.conforms?(schema, data)\n    true\n\n    iex> invalid_data = %{name: \"Alice\", age: \"thirty\"}\n    iex> Peri.conforms?(schema, invalid_data)\n    false","ref":"Peri.html#conforms?/3-examples"},{"type":"type","title":"Peri.custom_def/0","doc":"","ref":"Peri.html#t:custom_def/0"},{"type":"type","title":"Peri.default_def/0","doc":"","ref":"Peri.html#t:default_def/0"},{"type":"macro","title":"Peri.defschema/3","doc":"Defines a schema with a given name and schema definition.","ref":"Peri.html#defschema/3"},{"type":"macro","title":"Examples - Peri.defschema/3","doc":"defmodule MySchemas do\n      import Peri\n\n      defschema :user, %{\n        name: :string,\n        age: :integer,\n        email: {:required, :string}\n      }\n\n      # With permissive mode\n      defschema :flexible_user, %{\n        name: :string,\n        email: {:required, :string}\n      }, mode: :permissive\n    end\n\n    user_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    MySchemas.user(user_data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    MySchemas.user(invalid_data)\n    # => {:error, [email: \"is required\"]}\n\n    # Permissive mode preserves extra fields\n    flexible_data = %{name: \"John\", email: \"john@example.com\", role: \"admin\"}\n    MySchemas.flexible_user(flexible_data)\n    # => {:ok, %{name: \"John\", email: \"john@example.com\", role: \"admin\"}}","ref":"Peri.html#defschema/3-examples"},{"type":"type","title":"Peri.dependent_def/0","doc":"","ref":"Peri.html#t:dependent_def/0"},{"type":"type","title":"Peri.explicit_schema_def/0","doc":"","ref":"Peri.html#t:explicit_schema_def/0"},{"type":"type","title":"Peri.float_def/0","doc":"","ref":"Peri.html#t:float_def/0"},{"type":"function","title":"Peri.generate/1","doc":"Generates sample data based on the given schema definition using `StreamData`.\n\nThis function validates the schema first, and if the schema is valid, it uses the\n`Peri.Generatable.gen/1` function to generate data according to the schema.\n\nNote that this function returns a `Stream`, so you traverse easily the data generations.","ref":"Peri.html#generate/1"},{"type":"function","title":"Parameters - Peri.generate/1","doc":"- `schema`: The schema definition to generate data for.","ref":"Peri.html#generate/1-parameters"},{"type":"function","title":"Returns - Peri.generate/1","doc":"- `{:ok, stream}` if the data is successfully generated.\n  - `{:error, errors}` if there are validation errors in the schema.","ref":"Peri.html#generate/1-returns"},{"type":"function","title":"Examples - Peri.generate/1","doc":"iex> schema = %{name: :string, age: {:integer, {:range, {18, 65}}}}\n    iex> {:ok, stream} = Peri.generate(schema)\n    iex> [data] = Enum.take(stream, 1)\n    iex> is_map(data)\n    true\n    iex> data[:age] in 18..65\n    true","ref":"Peri.html#generate/1-examples"},{"type":"function","title":"Peri.get_enumerable_value/2","doc":"","ref":"Peri.html#get_enumerable_value/2"},{"type":"type","title":"Peri.int_def/0","doc":"","ref":"Peri.html#t:int_def/0"},{"type":"macro","title":"Peri.is_enumerable/1","doc":"Checks if the given data is an enumerable, specifically a map or a list.","ref":"Peri.html#is_enumerable/1"},{"type":"macro","title":"Parameters - Peri.is_enumerable/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_enumerable/1-parameters"},{"type":"macro","title":"Examples - Peri.is_enumerable/1","doc":"iex> is_enumerable(%{})\n    true\n\n    iex> is_enumerable([])\n    true\n\n    iex> is_enumerable(123)\n    false\n\n    iex> is_enumerable(\"string\")\n    false","ref":"Peri.html#is_enumerable/1-examples"},{"type":"macro","title":"Peri.is_numeric/1","doc":"Checks if the given data is a numeric value, specifically a integer or a float.","ref":"Peri.html#is_numeric/1"},{"type":"macro","title":"Parameters - Peri.is_numeric/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_numeric/1-parameters"},{"type":"macro","title":"Examples - Peri.is_numeric/1","doc":"iex> is_numeric(123)\n    true\n\n    iex> is_numeric(0xFF)\n    true\n\n    iex> is_numeric(12.12)\n    true\n\n    iex> is_numeric(\"string\")\n    false\n\n    iex> is_numeric(%{})\n    false","ref":"Peri.html#is_numeric/1-examples"},{"type":"macro","title":"Peri.is_numeric_type/1","doc":"Checks if the given type as an atom is a numeric (integer or float).","ref":"Peri.html#is_numeric_type/1"},{"type":"macro","title":"Parameters - Peri.is_numeric_type/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_numeric_type/1-parameters"},{"type":"macro","title":"Examples - Peri.is_numeric_type/1","doc":"iex> is_numeric(:integer)\n    true\n\n    iex> is_numeric(:float)\n    true\n\n    iex> is_numeric(:list)\n    false\n\n    iex> is_numeric({:enum, _})\n    false","ref":"Peri.html#is_numeric_type/1-examples"},{"type":"macro","title":"Peri.is_type_with_multiple_options/1","doc":"","ref":"Peri.html#is_type_with_multiple_options/1"},{"type":"type","title":"Peri.literal/0","doc":"","ref":"Peri.html#t:literal/0"},{"type":"type","title":"Peri.map_schema/0","doc":"","ref":"Peri.html#t:map_schema/0"},{"type":"type","title":"Peri.numeric_option/1","doc":"","ref":"Peri.html#t:numeric_option/1"},{"type":"function","title":"Peri.put_in_enum/3","doc":"Helper function to put a value into an enum, handling\nnot only maps and keyword lists but also structs.","ref":"Peri.html#put_in_enum/3"},{"type":"function","title":"Examples - Peri.put_in_enum/3","doc":"iex> Peri.put_in_enum(%{}, :hello, \"world\")\n    iex> Peri.put_in_enum(%{}, \"hello\", \"world\")\n    iex> Peri.put_in_enum(%User{}, :hello, \"world\")\n    iex> Peri.put_in_enum([], :hello, \"world\")","ref":"Peri.html#put_in_enum/3-examples"},{"type":"type","title":"Peri.schema/0","doc":"","ref":"Peri.html#t:schema/0"},{"type":"type","title":"Peri.schema_def/0","doc":"","ref":"Peri.html#t:schema_def/0"},{"type":"type","title":"Peri.string_def/0","doc":"","ref":"Peri.html#t:string_def/0"},{"type":"type","title":"Peri.string_option/0","doc":"","ref":"Peri.html#t:string_option/0"},{"type":"type","title":"Peri.time_def/0","doc":"","ref":"Peri.html#t:time_def/0"},{"type":"function","title":"Peri.to_changeset!/2","doc":"Converts a `Peri.schema()` definition to an Ecto [schemaless changesets](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-schemaless-changesets).","ref":"Peri.html#to_changeset!/2"},{"type":"type","title":"Peri.transform_def/0","doc":"","ref":"Peri.html#t:transform_def/0"},{"type":"function","title":"Peri.validate/3","doc":"Validates a given data map against a schema with options.\n\nReturns `{:ok, data}` if the data is valid according to the schema, or `{:error, errors}` if there are validation errors.","ref":"Peri.html#validate/3"},{"type":"function","title":"Parameters - Peri.validate/3","doc":"- schema: The schema definition map.\n  - data: The data map to be validated.\n  - opts: Options for validation.","ref":"Peri.html#validate/3-parameters"},{"type":"function","title":"Options - Peri.validate/3","doc":"- `:mode` - Validation mode. Can be `:strict` (default) or `:permissive`.\n    - `:strict` - Only fields defined in the schema are returned.\n    - `:permissive` - All fields from the input data are preserved.","ref":"Peri.html#validate/3-options"},{"type":"function","title":"Examples - Peri.validate/3","doc":"schema = %{name: :string, age: :integer}\n    data = %{name: \"John\", age: 30, extra: \"field\"}\n\n    # Strict mode (default)\n    Peri.validate(schema, data)\n    # => {:ok, %{name: \"John\", age: 30}}\n\n    # Permissive mode\n    Peri.validate(schema, data, mode: :permissive)\n    # => {:ok, %{name: \"John\", age: 30, extra: \"field\"}}","ref":"Peri.html#validate/3-examples"},{"type":"function","title":"Peri.validate_schema/1","doc":"Validates a schema definition to ensure it adheres to the expected structure and types.\n\nThis function can handle both simple and complex schema definitions, including nested schemas, custom validation functions, and various type constraints.","ref":"Peri.html#validate_schema/1"},{"type":"function","title":"Parameters - Peri.validate_schema/1","doc":"- `schema` - The schema definition to be validated. It can be a map or a keyword list representing the schema.","ref":"Peri.html#validate_schema/1-parameters"},{"type":"function","title":"Returns - Peri.validate_schema/1","doc":"- `{:ok, schema}` - If the schema is valid, returns the original schema.\n  - `{:error, errors}` - If the schema is invalid, returns an error tuple with detailed error information.","ref":"Peri.html#validate_schema/1-returns"},{"type":"function","title":"Examples - Peri.validate_schema/1","doc":"Validating a simple schema:\n\n  ```elixir\n  schema = %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Validating a nested schema:\n\n  ```elixir\n  schema = %{\n    user: %{\n      name: :string,\n      profile: %{\n        age: {:required, :integer},\n        email: {:required, :string}\n      }\n    }\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Handling invalid schema definition:\n\n  ```elixir\n  schema = %{\n    name: :str,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:error, _errors} = validate_schema(schema)\n  ```","ref":"Peri.html#validate_schema/1-examples"},{"type":"type","title":"Peri.validation/0","doc":"","ref":"Peri.html#t:validation/0"},{"type":"type","title":"Peri.validation_result/0","doc":"","ref":"Peri.html#t:validation_result/0"},{"type":"module","title":"Peri.Ecto.Type","doc":"Responsible to convert between Peri <> Ecto types definitions","ref":"Peri.Ecto.Type.html"},{"type":"function","title":"Peri.Ecto.Type.from/1","doc":"","ref":"Peri.Ecto.Type.html#from/1"},{"type":"module","title":"Peri.Ecto.Type.Any","doc":"Represents any type. Passes data to Cassandra as is.","ref":"Peri.Ecto.Type.Any.html"},{"type":"function","title":"Peri.Ecto.Type.Any.embed_as/1","doc":"Callback implementation for  Ecto.Type.embed_as/1 .","ref":"Peri.Ecto.Type.Any.html#embed_as/1"},{"type":"function","title":"Peri.Ecto.Type.Any.equal?/2","doc":"Callback implementation for  Ecto.Type.equal?/2 .","ref":"Peri.Ecto.Type.Any.html#equal?/2"},{"type":"module","title":"Peri.Ecto.Type.Atom","doc":"Custom Ecto type for storing atoms as strings","ref":"Peri.Ecto.Type.Atom.html"},{"type":"function","title":"Peri.Ecto.Type.Atom.embed_as/1","doc":"Callback implementation for  Ecto.Type.embed_as/1 .","ref":"Peri.Ecto.Type.Atom.html#embed_as/1"},{"type":"function","title":"Peri.Ecto.Type.Atom.equal?/2","doc":"Callback implementation for  Ecto.Type.equal?/2 .","ref":"Peri.Ecto.Type.Atom.html#equal?/2"},{"type":"module","title":"Peri.Ecto.Type.Either","doc":"Custom Ecto type for storing either of two types","ref":"Peri.Ecto.Type.Either.html"},{"type":"module","title":"Peri.Ecto.Type.OneOf","doc":"Custom Ecto type for storing one of many types, aka sum types","ref":"Peri.Ecto.Type.OneOf.html"},{"type":"module","title":"Peri.Ecto.Type.PID","doc":"Custom Ecto type for storing PIDs, use with caution","ref":"Peri.Ecto.Type.PID.html"},{"type":"function","title":"Peri.Ecto.Type.PID.embed_as/1","doc":"Callback implementation for  Ecto.Type.embed_as/1 .","ref":"Peri.Ecto.Type.PID.html#embed_as/1"},{"type":"function","title":"Peri.Ecto.Type.PID.equal?/2","doc":"Callback implementation for  Ecto.Type.equal?/2 .","ref":"Peri.Ecto.Type.PID.html#equal?/2"},{"type":"module","title":"Peri.Ecto.Type.Tuple","doc":"Custom Ecto type for storing tuples with multiple types/elements","ref":"Peri.Ecto.Type.Tuple.html"},{"type":"module","title":"Peri.Generatable","doc":"A module for generating sample data based on Peri schemas using StreamData.\n\nThis module provides functions to generate various types of data, conforming to the schema definitions given in Peri. It leverages the StreamData library to create streams of random data that match the specified types and constraints.","ref":"Peri.Generatable.html"},{"type":"module","title":"Examples - Peri.Generatable","doc":"iex> schema = %{\n    ...>   name: :string,\n    ...>   age: {:integer, {:gte, 18}},\n    ...>   active: :boolean\n    ...> }\n    iex> Peri.Generatable.gen(schema)\n    %StreamData{\n      type: :fixed_map,\n      data: %{name: StreamData.string(:alphanumeric), age: StreamData.filter(StreamData.integer(), &(&1 >= 18)), active: StreamData.boolean()}\n    }","ref":"Peri.Generatable.html#module-examples"},{"type":"function","title":"Peri.Generatable.gen/1","doc":"Generates a stream of data based on the given schema type.\n\nThis function provides various clauses to handle different types and constraints defined in Peri schemas. It uses StreamData to generate streams of random data conforming to the specified types and constraints.","ref":"Peri.Generatable.html#gen/1"},{"type":"function","title":"Parameters - Peri.Generatable.gen/1","doc":"- `schema`: The schema type to generate data for. It can be a simple type like `:integer`, `:string`, etc., or a complex type with constraints like `{:integer, {:gte, 18}}`.","ref":"Peri.Generatable.html#gen/1-parameters"},{"type":"function","title":"Returns - Peri.Generatable.gen/1","doc":"- A StreamData generator stream for the specified schema type.","ref":"Peri.Generatable.html#gen/1-returns"},{"type":"function","title":"Examples - Peri.Generatable.gen/1","doc":"iex> Peri.Generatable.gen(:atom)\n    %StreamData{type: :atom, data: ...}\n\n    iex> Peri.Generatable.gen(:string)\n    %StreamData{type: :string, data: ...}\n\n    iex> Peri.Generatable.gen(:integer)\n    %StreamData{type: :integer, data: ...}\n\n    iex> Peri.Generatable.gen({:enum, [:admin, :user, :guest]})\n    %StreamData{type: :one_of, data: ...}\n\n    iex> Peri.Generatable.gen({:list, :integer})\n    %StreamData{type: :list_of, data: ...}\n\n    iex> Peri.Generatable.gen({:tuple, [:string, :integer]})\n    %StreamData{type: :tuple, data: ...}\n\n    iex> Peri.Generatable.gen({:integer, {:gt, 10}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> Peri.Generatable.gen({:string, {:regex, ~r/^[a-z]+$/}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> Peri.Generatable.gen({:either, {:integer, :string}})\n    %StreamData{type: :one_of, data: ...}\n\n    iex> Peri.Generatable.gen({:custom, {MyModule, :my_fun}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> schema = %{name: :string, age: {:integer, {:gte, 18}}}\n    iex> Peri.Generatable.gen(schema)\n    %StreamData{type: :fixed_map, data: ...}","ref":"Peri.Generatable.html#gen/1-examples"},{"type":"module","title":"Peri.Parser","doc":"The `Peri.Parser` module is responsible for managing the state during schema validation. \nIt centralizes functions to handle updating data, adding errors, and managing the path within \nthe data structure being validated.","ref":"Peri.Parser.html"},{"type":"module","title":"Struct - Peri.Parser","doc":"The `Peri.Parser` struct has the following fields:\n\n- `:data` - The current state of the data being validated.\n- `:errors` - A list of errors encountered during validation.\n- `:path` - The current path within the data structure being validated.","ref":"Peri.Parser.html#module-struct"},{"type":"function","title":"Peri.Parser.add_error/2","doc":"Adds an error to the parser state's list of errors.","ref":"Peri.Parser.html#add_error/2"},{"type":"function","title":"Parameters - Peri.Parser.add_error/2","doc":"- `state` - The current `Peri.Parser` state.\n- `err` - The `%Peri.Error{}` struct representing the error to add.","ref":"Peri.Parser.html#add_error/2-parameters"},{"type":"function","title":"Examples - Peri.Parser.add_error/2","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> error = %Peri.Error{path: [:name], message: \"is required\", content: []}\n    iex> Peri.Parser.add_error(state, error)\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [%Peri.Error{path: [:name], message: \"is required\", content: []}], path: []}","ref":"Peri.Parser.html#add_error/2-examples"},{"type":"function","title":"Peri.Parser.for_list_element/3","doc":"Creates a new parser for a list element, preserving the root data.","ref":"Peri.Parser.html#for_list_element/3"},{"type":"function","title":"Parameters - Peri.Parser.for_list_element/3","doc":"- `element_data` - The data for the current list element.\n- `parent_parser` - The parent parser containing root data and path information.\n- `index` - The index of the element in the list.","ref":"Peri.Parser.html#for_list_element/3-parameters"},{"type":"function","title":"Examples - Peri.Parser.for_list_element/3","doc":"iex> parent = Peri.Parser.new(%{items: [1, 2, 3]}, root_data: %{items: [1, 2, 3]})\n    iex> Peri.Parser.for_list_element(1, parent, 0)\n    %Peri.Parser{data: 1, current_data: 1, root_data: %{items: [1, 2, 3]}, errors: [], path: [0]}","ref":"Peri.Parser.html#for_list_element/3-examples"},{"type":"function","title":"Peri.Parser.new/2","doc":"Initializes a new `Peri.Parser` struct with the given data.","ref":"Peri.Parser.html#new/2"},{"type":"function","title":"Parameters - Peri.Parser.new/2","doc":"- `data` - The initial data to be validated.","ref":"Peri.Parser.html#new/2-parameters"},{"type":"function","title":"Examples - Peri.Parser.new/2","doc":"iex> Peri.Parser.new(%{name: \"Alice\"})\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [], path: []}","ref":"Peri.Parser.html#new/2-examples"},{"type":"function","title":"Peri.Parser.update_data/3","doc":"Updates the data in the parser state at the given key with the specified value.","ref":"Peri.Parser.html#update_data/3"},{"type":"function","title":"Parameters - Peri.Parser.update_data/3","doc":"- `state` - The current `Peri.Parser` state.\n- `key` - The key to update in the data.\n- `val` - The value to set at the specified key.","ref":"Peri.Parser.html#update_data/3-parameters"},{"type":"function","title":"Examples - Peri.Parser.update_data/3","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> Peri.Parser.update_data(state, :age, 30)\n    %Peri.Parser{data: %{name: \"Alice\", age: 30}, errors: [], path: []}","ref":"Peri.Parser.html#update_data/3-examples"},{"type":"exception","title":"Peri.Error","doc":"Defines the structure and functions for handling validation errors in the Peri schema validation library.\n\nThe `Peri.Error` module encapsulates information about validation errors that occur during schema validation. Each error contains details about the path to the invalid data, the type of error, and any nested errors for complex or deeply nested schemas.","ref":"Peri.Error.html"},{"type":"exception","title":"Attributes - Peri.Error","doc":"- `:message` - A human-readable message describing the error.\n- `:content` - Additional information about the error, such as expected and actual values.\n- `:path` - A list representing the path to the invalid data within the structure being validated.\n- `:key` - The specific key or field that caused the error.\n- `:errors` - A list of nested `Peri.Error` structs for detailed information about nested validation errors.","ref":"Peri.Error.html#module-attributes"},{"type":"exception","title":"Example - Peri.Error","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    %Peri.Error{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %Peri.Error{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#module-example"},{"type":"exception","title":"Functions - Peri.Error","doc":"- `error_to_map/1` - Converts a `Peri.Error` struct to a map, including nested errors.","ref":"Peri.Error.html#module-functions"},{"type":"function","title":"Peri.Error.error_to_map/1","doc":"Recursively converts a `Peri.Error` struct into a map.","ref":"Peri.Error.html#error_to_map/1"},{"type":"function","title":"Parameters - Peri.Error.error_to_map/1","doc":"- `error` - A `Peri.Error` struct to be transformed.","ref":"Peri.Error.html#error_to_map/1-parameters"},{"type":"function","title":"Examples - Peri.Error.error_to_map/1","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    iex> Peri.Error.error_to_map(error)\n    %{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#error_to_map/1-examples"},{"type":"function","title":"Peri.Error.format_error_message/2","doc":"","ref":"Peri.Error.html#format_error_message/2"},{"type":"function","title":"Peri.Error.new_child/4","doc":"Creates a new child error with a path, key, message, and context.","ref":"Peri.Error.html#new_child/4"},{"type":"function","title":"Parameters - Peri.Error.new_child/4","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_child/4-parameters"},{"type":"function","title":"Examples - Peri.Error.new_child/4","doc":"iex> Peri.Error.new_child([:user], :age, \"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_child/4-examples"},{"type":"function","title":"Peri.Error.new_parent/3","doc":"Creates a new parent error with nested errors.","ref":"Peri.Error.html#new_parent/3"},{"type":"function","title":"Parameters - Peri.Error.new_parent/3","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `errors` - A list of nested `Peri.Error` structs.","ref":"Peri.Error.html#new_parent/3-parameters"},{"type":"function","title":"Examples - Peri.Error.new_parent/3","doc":"iex> Peri.Error.new_parent([:user], :age, [%Peri.Error{message: \"Invalid age\"}])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      errors: [%Peri.Error{message: \"Invalid age\"}]\n    }","ref":"Peri.Error.html#new_parent/3-examples"},{"type":"function","title":"Peri.Error.new_single/2","doc":"Creates a new single error with a formatted message and context.","ref":"Peri.Error.html#new_single/2"},{"type":"function","title":"Parameters - Peri.Error.new_single/2","doc":"- `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_single/2-parameters"},{"type":"function","title":"Examples - Peri.Error.new_single/2","doc":"iex> Peri.Error.new_single(\"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_single/2-examples"},{"type":"type","title":"Peri.Error.t/0","doc":"","ref":"Peri.Error.html#t:t/0"},{"type":"function","title":"Peri.Error.update_error_paths/2","doc":"","ref":"Peri.Error.html#update_error_paths/2"},{"type":"exception","title":"Peri.InvalidSchema","doc":"Exception raised when an invalid schema is encountered.\n\nThis exception is raised with a list of `Peri.Error` structs,\nproviding a readable message overview of the validation errors.","ref":"Peri.InvalidSchema.html"},{"type":"type","title":"Peri.InvalidSchema.t/0","doc":"","ref":"Peri.InvalidSchema.html#t:t/0"},{"type":"extras","title":"Peri","doc":"# Peri\n\nPeri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema. It provides a powerful and flexible way to define and validate schemas for your data, ensuring data integrity and consistency throughout your application.","ref":"readme.html"},{"type":"extras","title":"Features - Peri","doc":"- **Schema Definition**: Define schemas using a concise and expressive DSL\n- **Nested Validation**: Support for deeply nested and complex schemas\n- **Custom Validation**: Implement custom validation functions for specific requirements\n- **Data Generation**: Generate sample data based on your schemas using StreamData\n- **Ecto Integration**: Convert Peri schemas to Ecto changesets for seamless database integration\n- **Validation Modes**: Choose between strict (default) and permissive validation modes","ref":"readme.html#features"},{"type":"extras","title":"Installation - Peri","doc":"Add this line to your `mix.exs`:\n```elixir\ndefp deps do\n  [\n    {:peri, \"~> 0.5.1\"} # x-release-version\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - Peri","doc":"```elixir\ndefmodule MyApp.Schemas do\n  import Peri\n\n  defschema :user, %{\n    name: {:required, :string},\n    age: {:integer, {:gte, 18}},\n    email: {:required, :string},\n    role: {:enum, [:admin, :user, :guest]}\n  }\nend\n\n# Validate data\ndata = %{name: \"John\", age: 25, email: \"john@example.com\", role: :user}\nMyApp.Schemas.user(data)\n# => {:ok, validated_data}\n\n# Validate with permissive mode (preserves extra fields)\ndata_with_extra = %{name: \"John\", age: 25, email: \"john@example.com\", role: :user, extra: \"field\"}\nPeri.validate(MyApp.Schemas.get_schema(:user), data_with_extra, mode: :permissive)\n# => {:ok, %{name: \"John\", age: 25, email: \"john@example.com\", role: :user, extra: \"field\"}}\n```","ref":"readme.html#quick-start"},{"type":"extras","title":"Documentation - Peri","doc":"For detailed documentation on types, validation patterns, and integrations, see:\n\n- **[Types Reference](pages/types.md)** - All available types and constraints\n- **[Validation Patterns](pages/validation.md)** - Conditional, dependent, and custom validation  \n- **[Ecto Integration](pages/ecto.md)** - Converting schemas to Ecto changesets\n- **[Data Generation](pages/generation.md)** - Generate sample data with StreamData","ref":"readme.html#documentation"},{"type":"extras","title":"Why the Name \"Peri\"? - Peri","doc":"The name \"Peri\" is derived from the Greek word \"περί\" (pronounced \"peri\"), which means \"around\" or \"about.\" This name was chosen to reflect the library's primary purpose: to provide comprehensive and flexible schema validation for data structures in Elixir. Just as \"peri\" suggests encompassing or surrounding something, Peri aims to cover all aspects of data validation, ensuring that data conforms to specified rules and constraints.\n\nThe choice of the name \"Peri\" also hints at the library's ability to handle a wide variety of data types and structures, much like how the term \"around\" can denote versatility and inclusiveness. Whether it's validating nested maps, complex tuples, or strings with specific patterns, Peri is designed to be a robust tool that can adapt to various validation needs in Elixir programming.","ref":"readme.html#why-the-name-peri"},{"type":"extras","title":"Types Reference","doc":"# Types Reference\n\nPeri provides a comprehensive set of built-in types for schema validation.","ref":"types.html"},{"type":"extras","title":"Basic Types - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `:any` | Accepts any value | `:any` |\n| `:atom` | Validates atoms | `:atom` |\n| `:string` | Validates binary strings | `:string` |\n| `:integer` | Validates integers | `:integer` |\n| `:float` | Validates floats | `:float` |\n| `:boolean` | Validates booleans | `:boolean` |\n| `:map` | Validates maps (no content validation) | `:map` |\n| `:pid` | Validates process identifiers | `:pid` |","ref":"types.html#basic-types"},{"type":"extras","title":"Time Types - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `:date` | Validates `%Date{}` | `:date` |\n| `:time` | Validates `%Time{}` | `:time` |\n| `:datetime` | Validates `%DateTime{}` | `:datetime` |\n| `:naive_datetime` | Validates `%NaiveDateTime{}` | `:naive_datetime` |\n| `:duration` | Validates `%Duration{}` | `:duration` |","ref":"types.html#time-types"},{"type":"extras","title":"Collection Types - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:list, type}` | List of elements of specified type | `{:list, :string}` |\n| `{:map, type}` | Map with values of specified type | `{:map, :integer}` |\n| `{:map, key_type, value_type}` | Map with typed keys and values | `{:map, :atom, :string}` |\n| `{:tuple, types}` | Tuple with elements of specified types | `{:tuple, [:float, :float]}` |\n| `{:schema, map_schema, {:additional_keys, type}}` | Map with some strictly defined fields, with extras under a different type | `{:schema, %{main: :string}, {:additional_keys, :integer}}` |","ref":"types.html#collection-types"},{"type":"extras","title":"String Constraints - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:string, {:regex, regex}}` | String matching regex pattern | `{:string, {:regex, ~r/^\\w+$/}}` |\n| `{:string, {:eq, value}}` | String equal to value | `{:string, {:eq, \"exact\"}}` |\n| `{:string, {:min, length}}` | String with minimum length | `{:string, {:min, 3}}` |\n| `{:string, {:max, length}}` | String with maximum length | `{:string, {:max, 50}}` |\n| `{:string, [...options]}` | String with multiple options | `{:string, [min: 8, max: 64, regex: ~r/^[a-zA-Z0-9-]+$/]}` |","ref":"types.html#string-constraints"},{"type":"extras","title":"Integer Constraints - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:integer, {:eq, value}}` | Integer equal to value | `{:integer, {:eq, 42}}` |\n| `{:integer, {:neq, value}}` | Integer not equal to value | `{:integer, {:neq, 0}}` |\n| `{:integer, {:gt, value}}` | Integer greater than value | `{:integer, {:gt, 0}}` |\n| `{:integer, {:gte, value}}` | Integer greater than or equal | `{:integer, {:gte, 18}}` |\n| `{:integer, {:lt, value}}` | Integer less than value | `{:integer, {:lt, 100}}` |\n| `{:integer, {:lte, value}}` | Integer less than or equal | `{:integer, {:lte, 99}}` |\n| `{:integer, {:range, {min, max}}}` | Integer within range (inclusive) | `{:integer, {:range, {18, 65}}}` |\n| `{:integer, [...options]}` | Integer with multiple options | `{:integer, [gt: 12, lte: 96]}` |","ref":"types.html#integer-constraints"},{"type":"extras","title":"Float Constraints - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:float, {:eq, value}}` | Float equal to value | `{:float, {:eq, 3.1415}}` |\n| `{:float, {:neq, value}}` | Float not equal to value | `{:float, {:neq, 1.9}}` |\n| `{:float, {:gt, value}}` | Float greater than value | `{:float, {:gt, 1.0}}` |\n| `{:float, {:gte, value}}` | Float greater than or equal | `{:float, {:gte, 9.12}}` |\n| `{:float, {:lt, value}}` | Float less than value | `{:float, {:lt, 10.0}}` |\n| `{:float, {:lte, value}}` | Float less than or equal | `{:float, {:lte, 99.999}}` |\n| `{:float, {:range, {min, max}}}` | Float within range (inclusive) | `{:float, {:range, {8.3, 15.3}}}` |\n| `{:float, [...options]}` | Float with multiple options | `{:float, [gt: 1.52, lte: 29.123]}` |","ref":"types.html#float-constraints"},{"type":"extras","title":"Choice Types - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:enum, choices}` | Value must be one of choices | `{:enum, [:admin, :user, :guest]}` |\n| `{:literal, value}` | Value must exactly match | `{:literal, :active}` |\n| `{:either, {type1, type2}}` | Value matches either type | `{:either, {:string, :integer}}` |\n| `{:oneof, types}` | Value matches one of types | `{:oneof, [:string, :integer, :atom]}` |","ref":"types.html#choice-types"},{"type":"extras","title":"Modifiers - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:required, type}` | Field is required | `{:required, :string}` |\n| `{type, {:default, value}}` | Default value if missing | `{:string, {:default, \"unknown\"}}` |\n| `{type, {:default, fun}}` | Default from function | `{:integer, {:default, &System.system_time/0}}` |\n| `{type, {:default, {m, f}}}` | Default from MFA | `{:string, {:default, {MyMod, :get_default}}}` |\n| `{type, {:transform, fun}}` | Transform value | `{:string, {:transform, &String.upcase/1}}` |\n| `{type, {:transform, {m, f}}}` | Transform with MFA | `{:string, {:transform, {MyMod, :clean}}}` |","ref":"types.html#modifiers"},{"type":"extras","title":"Custom Validation - Types Reference","doc":"| Type | Description | Example |\n|------|-------------|---------|\n| `{:custom, callback}` | Custom validation function | `{:custom, &validate_email/1}` |\n| `{:custom, {mod, fun}}` | Custom validation MFA | `{:custom, {MyMod, :validate}}` |\n| `{:custom, {mod, fun, args}}` | Custom validation with args | `{:custom, {MyMod, :validate, []}}` |","ref":"types.html#custom-validation"},{"type":"extras","title":"Examples - Types Reference","doc":"","ref":"types.html#examples"},{"type":"extras","title":"Simple User Schema - Types Reference","doc":"```elixir\n%{\n  name: {:required, :string},\n  age: {:integer, {:gte, 0}},\n  email: {:required, {:string, {:regex, ~r/@/}}},\n  role: {:enum, [:admin, :user]}\n}\n```","ref":"types.html#simple-user-schema"},{"type":"extras","title":"Complex Nested Schema - Types Reference","doc":"```elixir\n%{\n  user: %{\n    profile: {:required, %{\n      name: {:required, :string},\n      bio: {:string, {:max, 500}}\n    }},\n    preferences: {:map, :string, :boolean},\n    tags: {:list, :string}\n  }\n}\n```","ref":"types.html#complex-nested-schema"},{"type":"extras","title":"Validation Patterns","doc":"# Validation Patterns\n\nAdvanced validation patterns for dynamic and context-aware schemas.","ref":"validation.html"},{"type":"extras","title":"Validation Modes - Validation Patterns","doc":"Peri supports two validation modes to control how extra fields are handled:","ref":"validation.html#validation-modes"},{"type":"extras","title":"Strict Mode (Default) - Validation Patterns","doc":"By default, Peri operates in strict mode, which filters out any fields not defined in the schema:\n\n```elixir\nschema = %{\n  name: :string,\n  age: :integer\n}\n\ndata = %{name: \"John\", age: 30, extra: \"field\"}\n\n{:ok, result} = Peri.validate(schema, data)\n# result => %{name: \"John\", age: 30}\n```","ref":"validation.html#strict-mode-default"},{"type":"extras","title":"Permissive Mode - Validation Patterns","doc":"Permissive mode preserves all fields from the input data, even those not defined in the schema:\n\n```elixir\n{:ok, result} = Peri.validate(schema, data, mode: :permissive)\n# result => %{name: \"John\", age: 30, extra: \"field\"}\n```","ref":"validation.html#permissive-mode"},{"type":"extras","title":"Using Permissive Mode with defschema - Validation Patterns","doc":"You can define schemas that always use permissive mode:\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  # Strict mode (default)\n  defschema :user_strict, %{\n    name: :string,\n    email: {:required, :string}\n  }\n\n  # Permissive mode\n  defschema :user_permissive, %{\n    name: :string,\n    email: {:required, :string}\n  }, mode: :permissive\nend\n\ndata = %{name: \"John\", email: \"john@example.com\", role: \"admin\"}\n\n# Strict mode filters out 'role'\n{:ok, strict} = MySchemas.user_strict(data)\n# strict => %{name: \"John\", email: \"john@example.com\"}\n\n# Permissive mode keeps 'role'\n{:ok, permissive} = MySchemas.user_permissive(data)  \n# permissive => %{name: \"John\", email: \"john@example.com\", role: \"admin\"}\n```","ref":"validation.html#using-permissive-mode-with-defschema"},{"type":"extras","title":"Use Cases for Permissive Mode - Validation Patterns","doc":"Permissive mode is useful when:\n- Building API gateways that need to forward extra fields\n- Implementing progressive validation in layers\n- Working with evolving data structures where new fields may be added\n- Creating middleware that validates known fields but passes through metadata\n\n**Note**: Fields not defined in the schema are not validated, they are simply passed through unchanged.","ref":"validation.html#use-cases-for-permissive-mode"},{"type":"extras","title":"Conditional Validation - Validation Patterns","doc":"Use `:cond` to validate fields based on runtime conditions.\n\n```elixir\ndefmodule UserSchema do\n  import Peri\n\n  defschema :registration, %{\n    name: {:required, :string},\n    is_premium: {:required, :boolean},\n    # Only require payment info if premium\n    payment_info: {:cond, & &1.is_premium, {:required, :string}, nil}\n  }\nend\n```","ref":"validation.html#conditional-validation"},{"type":"extras","title":"Dependent Validation - Validation Patterns","doc":"","ref":"validation.html#dependent-validation"},{"type":"extras","title":"Single Field Dependency - Validation Patterns","doc":"Validate a field based on another field's value:\n\n```elixir\ndefmodule AuthSchema do\n  import Peri\n\n  defschema :user, %{\n    password: {:required, :string},\n    password_confirmation: {:dependent, :password, &match_passwords/2, :string}\n  }\n\n  defp match_passwords(password, password), do: :ok\n  defp match_passwords(_, _), do: {:error, \"passwords must match\", []}\nend\n```","ref":"validation.html#single-field-dependency"},{"type":"extras","title":"Multiple Field Dependencies - Validation Patterns","doc":"Complex validation based on multiple fields:\n\n```elixir\ndefmodule ProfileSchema do\n  import Peri\n\n  defschema :user_profile, %{\n    contact_type: {:required, {:enum, [:email, :phone, :both]}},\n    email: {:string, {:regex, ~r/@/}},\n    phone: :string,\n    contact_info: {:dependent, &validate_contact/1}\n  }\n\n  defp validate_contact(%{data: %{contact_type: :email}}) do\n    {:ok, {:required, %{email: {:required, :string}}}}\n  end\n  \n  defp validate_contact(%{data: %{contact_type: :phone}}) do\n    {:ok, {:required, %{phone: {:required, :string}}}}\n  end\n  \n  defp validate_contact(%{data: %{contact_type: :both}}) do\n    {:ok, {:required, %{\n      email: {:required, :string},\n      phone: {:required, :string}\n    }}}\n  end\nend\n```","ref":"validation.html#multiple-field-dependencies"},{"type":"extras","title":"Custom Validation - Validation Patterns","doc":"","ref":"validation.html#custom-validation"},{"type":"extras","title":"Simple Custom Validator - Validation Patterns","doc":"```elixir\ndefmodule ProductSchema do\n  import Peri\n\n  defschema :product, %{\n    price: {:custom, &validate_price/1}\n  }\n\n  defp validate_price(price) when price > 0, do: :ok\n  defp validate_price(price), do: {:error, \"price must be positive, got %{price}\", [price: price]}\nend\n```","ref":"validation.html#simple-custom-validator"},{"type":"extras","title":"MFA Custom Validators - Validation Patterns","doc":"```elixir\ndefmodule OrderSchema do\n  import Peri\n\n  defschema :order, %{\n    items: {:list, {:custom, {__MODULE__, :validate_item, [:in_stock]}}},\n    total: {:custom, {Calculator, :validate_total}}\n  }\n\n  def validate_item(item, :in_stock) do\n    if item.stock > 0 do\n      :ok\n    else\n      {:error, \"item %{name} is out of stock\", [name: item.name]}\n    end\n  end\nend\n```","ref":"validation.html#mfa-custom-validators"},{"type":"extras","title":"Callback Arities - Validation Patterns","doc":"","ref":"validation.html#callback-arities"},{"type":"extras","title":"1-Arity Callbacks (Root Data) - Validation Patterns","doc":"Receives the entire root data structure:\n\n```elixir\n%{\n  user_type: :premium,\n  features: {:cond, fn data -> data.user_type == :premium end, \n             {:list, :string}, \n             {:literal, []}}\n}\n```","ref":"validation.html#1-arity-callbacks-root-data"},{"type":"extras","title":"2-Arity Callbacks (Current + Root) - Validation Patterns","doc":"Receives current context and root data - useful for lists:\n\n```elixir\ndefmodule ItemSchema do\n  import Peri\n\n  defschema :item, %{\n    type: {:required, :string},\n    # Validates based on current item's type, not parent data\n    value: {:dependent, fn current, _root ->\n      case current.type do\n        \"number\" -> {:ok, :integer}\n        \"text\" -> {:ok, :string}\n        _ -> {:ok, :any}\n      end\n    end}\n  }\n\n  defschema :collection, %{\n    items: {:list, get_schema(:item)}\n  }\nend\n\n# Each item validates independently\ndata = %{\n  items: [\n    %{type: \"number\", value: 42},\n    %{type: \"text\", value: \"hello\"}\n  ]\n}\n```","ref":"validation.html#2-arity-callbacks-current-root"},{"type":"extras","title":"Schema Composition - Validation Patterns","doc":"","ref":"validation.html#schema-composition"},{"type":"extras","title":"Reusable Schemas - Validation Patterns","doc":"```elixir\ndefmodule BaseSchemas do\n  import Peri\n\n  defschema :address, %{\n    street: {:required, :string},\n    city: {:required, :string},\n    zip: {:string, {:regex, ~r/^\\d{5}$/}}\n  }\n\n  defschema :person, %{\n    name: {:required, :string},\n    address: get_schema(:address)\n  }\n\n  defschema :company, %{\n    name: {:required, :string},\n    headquarters: get_schema(:address),\n    employees: {:list, get_schema(:person)}\n  }\nend\n```","ref":"validation.html#reusable-schemas"},{"type":"extras","title":"Schema Merging - Validation Patterns","doc":"```elixir\ndefmodule UserSchemas do\n  import Peri\n\n  defschema :base_user, %{\n    name: {:required, :string},\n    email: {:required, :string}\n  }\n\n  defschema :admin_user, Map.merge(get_schema(:base_user), %{\n    permissions: {:required, {:list, :string}},\n    last_login: :datetime\n  })\nend\n```","ref":"validation.html#schema-merging"},{"type":"extras","title":"Error Context - Validation Patterns","doc":"Custom validators can provide detailed error context:\n\n```elixir\ndefp validate_complex_rule(value) do\n  case expensive_validation(value) do\n    :ok -> :ok\n    {:error, reason} -> \n      {:error, \"validation failed: %{reason} for value %{value}\", \n       [reason: reason, value: inspect(value)]}\n  end\nend\n```","ref":"validation.html#error-context"},{"type":"extras","title":"Ecto Integration","doc":"# Ecto Integration\n\nConvert Peri schemas to Ecto schemaless changesets for data validation.","ref":"ecto.html"},{"type":"extras","title":"Basic Usage - Ecto Integration","doc":"```elixir\ndefmodule UserValidator do\n  import Peri\n\n  defschema :user, %{\n    name: {:required, :string},\n    age: {:integer, {:gte, 18}},\n    email: {:required, {:string, {:regex, ~r/@/}}},\n    role: {:enum, [:admin, :user]}\n  }\n\n  def validate_user(attrs) do\n    # Convert Peri schema to Ecto changeset\n    Peri.to_changeset!(get_schema(:user), attrs)\n  end\nend\n\n# Usage\nattrs = %{\"name\" => \"John\", \"age\" => 25, \"email\" => \"john@example.com\"}\nchangeset = UserValidator.validate_user(attrs)\n```","ref":"ecto.html#basic-usage"},{"type":"extras","title":"Type Mapping - Ecto Integration","doc":"Peri types are automatically mapped to appropriate Ecto types:\n\n| Peri Type | Ecto Type | Notes |\n|-----------|-----------|--------|\n| `:string` | `:string` | Direct mapping |\n| `:integer` | `:integer` | Direct mapping |\n| `:float` | `:float` | Direct mapping |\n| `:boolean` | `:boolean` | Direct mapping |\n| `:date` | `:date` | Direct mapping |\n| `:datetime` | `:utc_datetime` | UTC datetime |\n| `:naive_datetime` | `:naive_datetime` | Direct mapping |\n| `{:enum, choices}` | Custom validation | Validates against choices |\n| `{:list, type}` | `{:array, type}` | Array of specified type |\n| `:any` | Custom type | Allows any value |\n| `:atom` | Custom type | Validates atoms |\n| `{:tuple, types}` | Custom type | Validates tuple structure |","ref":"ecto.html#type-mapping"},{"type":"extras","title":"Constraint Mapping - Ecto Integration","doc":"Peri constraints are converted to Ecto validations:\n\n```elixir\n# Peri schema\n%{\n  name: {:string, {:min, 2}},\n  age: {:integer, {:range, {18, 65}}},\n  email: {:string, {:regex, ~r/@/}}\n}\n\n# Equivalent Ecto validations applied\nchangeset\n|> validate_length(:name, min: 2)\n|> validate_number(:age, greater_than_or_equal_to: 18, less_than_or_equal_to: 65)\n|> validate_format(:email, ~r/@/)\n```","ref":"ecto.html#constraint-mapping"},{"type":"extras","title":"Nested Validation - Ecto Integration","doc":"```elixir\ndefmodule ProfileValidator do\n  import Peri\n\n  defschema :address, %{\n    street: {:required, :string},\n    city: {:required, :string},\n    zip: {:string, {:regex, ~r/^\\d{5}$/}}\n  }\n\n  defschema :user, %{\n    name: {:required, :string},\n    address: {:required, get_schema(:address)},\n    tags: {:list, :string}\n  }\n\n  def validate_user(attrs) do\n    changeset = Peri.to_changeset!(get_schema(:user), attrs)\n    # Nested validation happens automatically\n    case changeset.valid? do\n      true -> {:ok, Ecto.Changeset.apply_changes(changeset)}\n      false -> {:error, changeset}\n    end\n  end\nend\n\n# Usage with nested data\nattrs = %{\n  \"name\" => \"John\",\n  \"address\" => %{\n    \"street\" => \"123 Main St\",\n    \"city\" => \"Anytown\",\n    \"zip\" => \"12345\"\n  },\n  \"tags\" => [\"developer\", \"elixir\"]\n}\n\ncase ProfileValidator.validate_user(attrs) do\n  {:ok, validated_data} -> IO.puts(\"Valid!\")\n  {:error, changeset} -> IO.inspect(changeset.errors)\nend\n```","ref":"ecto.html#nested-validation"},{"type":"extras","title":"Custom Types - Ecto Integration","doc":"Peri provides custom Ecto types for advanced validation:\n\n```elixir\n# Available custom types\nPeri.Ecto.Type.Any      # Accepts any value\nPeri.Ecto.Type.Atom     # Validates atoms\nPeri.Ecto.Type.Tuple    # Validates tuples\nPeri.Ecto.Type.Either   # Union types\nPeri.Ecto.Type.OneOf    # Multiple choice types\n```","ref":"ecto.html#custom-types"},{"type":"extras","title":"Error Handling - Ecto Integration","doc":"Peri validation errors are automatically converted to Ecto changeset errors:\n\n```elixir\nattrs = %{\"name\" => \"\", \"age\" => 15}\nchangeset = Peri.to_changeset!(schema, attrs)\n\n# Access errors like normal Ecto changeset\nchangeset.errors\n# [\n#   name: {\"can't be blank\", [validation: :required]}, \n#   age: {\"must be greater than or equal to %{number}\", \n#         [validation: :number, kind: :greater_than_or_equal_to, number: 18]}\n# ]\n\n# Check if valid\nif changeset.valid? do\n  data = Ecto.Changeset.apply_changes(changeset)\n  {:ok, data}\nelse\n  {:error, changeset}\nend\n```","ref":"ecto.html#error-handling"},{"type":"extras","title":"Working with Phoenix - Ecto Integration","doc":"Perfect for Phoenix controller validation:\n\n```elixir\ndefmodule MyAppWeb.UserController do\n  use MyAppWeb, :controller\n  import Peri\n\n  defschema :user_params, %{\n    name: {:required, :string},\n    email: {:required, {:string, {:regex, ~r/@/}}},\n    age: {:integer, {:gte, 18}}\n  }\n\n  def create(conn, params) do\n    changeset = Peri.to_changeset!(get_schema(:user_params), params)\n    \n    if changeset.valid? do\n      user_data = Ecto.Changeset.apply_changes(changeset)\n      # Process valid data...\n      json(conn, %{success: true, user: user_data})\n    else\n      conn\n      |> put_status(:unprocessable_entity)\n      |> json(%{errors: translate_errors(changeset)})\n    end\n  end\n\n  defp translate_errors(changeset) do\n    Ecto.Changeset.traverse_errors(changeset, fn {msg, opts} ->\n      Regex.replace(~r\"%{(\\w+)}\", msg, fn _, key ->\n        opts |> Keyword.get(String.to_existing_atom(key), key) |> to_string()\n      end)\n    end)\n  end\nend\n```","ref":"ecto.html#working-with-phoenix"},{"type":"extras","title":"Benefits - Ecto Integration","doc":"- **Familiar API**: Uses standard Ecto changeset interface\n- **Rich Validation**: Access to all Peri's validation features\n- **Error Consistency**: Standard Ecto error format\n- **Phoenix Ready**: Works seamlessly with Phoenix forms and APIs\n- **Composable**: Combine with other Ecto changeset operations","ref":"ecto.html#benefits"},{"type":"extras","title":"Data Generation","doc":"# Data Generation\n\nGenerate sample data based on your Peri schemas using StreamData.","ref":"generation.html"},{"type":"extras","title":"Setup - Data Generation","doc":"Add StreamData to your dependencies:\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:peri, \"~> 0.5.1\"}, # x-release-version\n    {:stream_data, \"~> 1.0\", only: [:test, :dev]}\n  ]\nend\n```","ref":"generation.html#setup"},{"type":"extras","title":"Basic Usage - Data Generation","doc":"```elixir\ndefmodule UserSchema do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: {:integer, {:gte, 18}},\n    active: :boolean\n  }\nend\n\n# Generate sample data\nsample_data = Peri.generate(UserSchema.get_schema(:user))\nEnum.take(sample_data, 3)\n# [\n#   %{name: \"abc\", age: 23, active: true},\n#   %{name: \"xyz\", age: 45, active: false},\n#   %{name: \"def\", age: 67, active: true}\n# ]\n```","ref":"generation.html#basic-usage"},{"type":"extras","title":"Constraint-Aware Generation - Data Generation","doc":"Generators respect Peri constraints:\n\n```elixir\nschema = %{\n  username: {:string, {:regex, ~r/^[a-z]+$/}},\n  score: {:integer, {:range, {0, 100}}},\n  role: {:enum, [:admin, :user, :guest]},\n  tags: {:list, :string}\n}\n\n# Generated data will match constraints\nsamples = Peri.generate(schema) |> Enum.take(5)\n# All usernames will be lowercase letters only\n# All scores will be 0-100\n# All roles will be from the enum\n```","ref":"generation.html#constraint-aware-generation"},{"type":"extras","title":"Property-Based Testing - Data Generation","doc":"Perfect for property-based testing with ExUnit:\n\n```elixir\ndefmodule UserTest do\n  use ExUnit.Case\n  use ExUnitProperties\n  import Peri\n\n  defschema :user, %{\n    name: {:required, :string},\n    age: {:integer, {:gte, 0, :lte, 120}},\n    email: {:string, {:regex, ~r/@/}}\n  }\n\n  property \"user validation always succeeds for generated data\" do\n    check all user_data <- Peri.generate(get_schema(:user)) do\n      assert {:ok, _} = Peri.validate(get_schema(:user), user_data)\n    end\n  end\n\n  property \"age is always within bounds\" do\n    check all %{age: age} <- Peri.generate(get_schema(:user)) do\n      assert age >= 0 and age <= 120\n    end\n  end\nend\n```","ref":"generation.html#property-based-testing"},{"type":"extras","title":"Complex Data Structures - Data Generation","doc":"Generate nested and complex data:\n\n```elixir\ndefmodule ComplexSchema do\n  import Peri\n\n  defschema :address, %{\n    street: :string,\n    city: :string,\n    zip: {:string, {:regex, ~r/^\\d{5}$/}}\n  }\n\n  defschema :person, %{\n    name: {:required, :string},\n    addresses: {:list, get_schema(:address)},\n    preferences: {:map, :string, :boolean},\n    coordinates: {:tuple, [:float, :float]}\n  }\nend\n\n# Generates fully nested structures\nsample = Peri.generate(ComplexSchema.get_schema(:person)) |> Enum.take(1) |> hd()\n# %{\n#   name: \"John\",\n#   addresses: [\n#     %{street: \"123 Main\", city: \"Boston\", zip: \"02101\"},\n#     %{street: \"456 Oak\", city: \"Salem\", zip: \"01970\"}\n#   ],\n#   preferences: %{\"dark_mode\" => true, \"notifications\" => false},\n#   coordinates: {42.3601, -71.0589}\n# }\n```","ref":"generation.html#complex-data-structures"},{"type":"extras","title":"Custom Generators - Data Generation","doc":"For `:custom` types, provide your own generators:\n\n```elixir\ndefmodule CustomSchema do\n  import Peri\n\n  defschema :product, %{\n    id: {:custom, &validate_uuid/1},\n    price: {:custom, &validate_price/1}\n  }\n\n  defp validate_uuid(uuid) when is_binary(uuid) do\n    # UUID validation logic\n    :ok\n  end\n\n  defp validate_price(price) when is_number(price) and price > 0, do: :ok\n  defp validate_price(_), do: {:error, \"invalid price\", []}\nend\n\n# Note: Custom generators need to be implemented separately\n# This is a limitation - custom types require manual generator setup\n```","ref":"generation.html#custom-generators"},{"type":"extras","title":"Seeding Development Data - Data Generation","doc":"Use for development database seeding:\n\n```elixir\ndefmodule MyApp.Seeds do\n  import Peri\n\n  defschema :user_seed, %{\n    name: {:string, {:min, 3}},\n    email: {:string, {:regex, ~r/.+@.+\\..+/}},\n    role: {:enum, [:admin, :user]},\n    active: {:boolean, {:default, true}}\n  }\n\n  def seed_users(count \\\\ 50) do\n    Peri.generate(get_schema(:user_seed))\n    |> Enum.take(count)\n    |> Enum.each(fn user_data ->\n      MyApp.Users.create_user(user_data)\n    end)\n  end\nend\n\n# Run in seeds.exs or IEx\nMyApp.Seeds.seed_users(100)\n```","ref":"generation.html#seeding-development-data"},{"type":"extras","title":"API Testing - Data Generation","doc":"Generate test data for API endpoints:\n\n```elixir\ndefmodule MyAppWeb.UserControllerTest do\n  use MyAppWeb.ConnCase\n  use ExUnitProperties\n  import Peri\n\n  defschema :user_request, %{\n    name: {:required, :string},\n    email: {:required, {:string, {:regex, ~r/@/}}},\n    age: {:integer, {:gte, 18}}\n  }\n\n  property \"POST /users accepts valid user data\" do\n    check all user_params <- Peri.generate(get_schema(:user_request)) do\n      conn = post(build_conn(), \"/users\", user_params)\n      assert json_response(conn, 201)\n    end\n  end\nend\n```","ref":"generation.html#api-testing"},{"type":"extras","title":"Limitations - Data Generation","doc":"- **Custom Types**: Custom validation types need manual generator implementation\n- **Performance**: Large data generation may be slow for complex nested structures\n- **Dependencies**: Requires StreamData dependency for generation features","ref":"generation.html#limitations"},{"type":"extras","title":"Best Practices - Data Generation","doc":"- Use generation primarily for testing and development seeding\n- Keep generated data size reasonable for performance\n- Combine with ExUnitProperties for comprehensive property-based testing\n- Use constraints to ensure realistic data generation","ref":"generation.html#best-practices"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}